#!/usr/bin/python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: Ask for help
# Based on User Info by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# 27.4.2017
# Modified from User Info by Niko-Ville Koljonen
# 01/2023

import gi
gi.require_version("Gtk", "3.0")
from gi.repository import Gtk as gtk
from gi.repository import Gdk
import subprocess
import os
import re
import pwd
import datetime

class InfoWin:

    def create_list(self):
        self.username = pwd.getpwuid(os.getuid()).pw_name
        try:
            userinfo_cache = subprocess.check_output("ldapsearch -o nettimeout=3 cn=" + self.username + " pwdLastSet displayName mail accountExpires homeDirectory homeDrive msExchRecipientDisplayType msExchRecipientTypeDetails msDS-UserPasswordExpiryTimeComputed", shell=True)
            userinfo_cache = userinfo_cache.decode("utf-8")
            self.info = re.search("(# " + self.username + ".*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)", userinfo_cache, re.M)
        except Exception:
            userinfo_cache = subprocess.check_output("cat /home/" + self.username + "/.cache/usercache", shell=True).decode("utf-8")
            self.info = re.search("(# " + self.username + ".*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)\n(.*)", userinfo_cache, re.M)
        return self.info


    def __init__(self):

        # get the user information from cache or AD to a variable
        self.info = self.create_list()

        # Username
        self.username = pwd.getpwuid(os.getuid()).pw_name
        self.username_text = "Username: "

        # Full name
        self.fullname = re.search("\s.*", self.info.group(3)).group(0).strip()
        self.fullname_text = "Full name: "

	# Computer name
        self.computer_name = subprocess.check_output('uname -n', shell = True).decode('utf-8').rstrip("\n")
        self.computer_name_text = "Computer: "
        print (self.computer_name)

	# Ip address for remote help
#        ip_addr = subprocess.run("ip a | grep tun | grep 115 | cut -d '/' -f 1 | sed -e 's/inet//g' -e 's/ //g'", shell = True, stdout=subprocess.PIPE)
        self.help_ip = subprocess.check_output("ip a | grep tun | grep inet | cut -d '/' -f 1 | sed -e 's/inet//g' -e 's/ //g' | grep 115", shell = True).decode('utf-8').rstrip("\n")
        self.help_ip_text = "For administrator access: "
        print (self.help_ip)

        # ANSI standard
        ansistandard = datetime.datetime(1601, 1, 1)

        # Create a new window
        self.window = gtk.Window(gtk.WindowType.TOPLEVEL)
        self.window.set_title("Information for admins")

        # Close the window
        self.window.connect("destroy", lambda w: gtk.main_quit())

        # Size of the window
        self.window.set_size_request(470, 340)
        #self.window.set_position(gtk.WindowType.CENTER)

        self.vbox = gtk.VBox(False, 0)
        self.window.add(self.vbox)
        self.window.set_border_width(10)

        # Create a table for the labels
        table = gtk.Table(5, 2, True)

        # Attach labels to the table
        table.attach(self.create_label(self.username_text),      0, 1, 0, 1, ypadding=5)
        table.attach(self.create_label(self.username),           1, 2, 0, 1, ypadding=5)
        table.attach(self.create_label(self.fullname_text),      0, 1, 1, 2, ypadding=5)
        table.attach(self.create_label(self.fullname),           1, 2, 1, 2, ypadding=5)
        table.attach(self.create_label(self.computer_name_text), 0, 1, 2, 3, ypadding=5)
        table.attach(self.create_label(self.computer_name),      1, 2, 2, 3, ypadding=5)
        table.attach(self.create_label(self.help_ip_text),       0, 1, 3, 4, ypadding=5)
        table.attach(self.create_label(self.help_ip),            1, 2, 3, 4, ypadding=5)
        table.set_homogeneous(False)

        # Show the table
        self.vbox.pack_start(table, True, True, 0)

        # Clipboard
        self.clipboard = gtk.Clipboard.get(Gdk.SELECTION_CLIPBOARD)

        # Copy button
        self.button = gtk.Button("Copy the info to clipboard")
        self.button.connect("clicked", self.set_clipboard)
        self.button_fixed = gtk.Fixed()
        self.button_fixed.put(self.button, 130, 10)
        self.vbox.add(self.button_fixed)
        self.button_fixed.show()

        # Shows the window
        self.window.show_all()

    def create_label(self, content1):

        label = gtk.Label(content1)
        label.set_alignment(0, 1)
        return label

    def set_clipboard(self, widget):
        self.clipboard.set_text(self.username_text + self.username + "\n" + self.fullname_text + self.fullname + "\n" + self.computer_name_text + self.computer_name + "\n" + self.help_ip_text + self.help_ip, -1)
        self.clipboard.store()
        return

def main():
    gtk.main()
    return 0

if __name__ == "__main__":
    InfoWin()
    main()
