#!/usr/bin/env python3

# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

# Program: HY-menu
# Written by Antti Luukkonen <antti.luukkonen@helsinki.fi>
# Small changes by Jani Jaakkola <jani.jaakkola@helsinki.fi>
# 18.10.2018 and ported to Python 3 by JJ 23.7.2020.

# Removing deprecated calls and migrate to emojis + reboot checks
# 31.05.2022 by N-V Koljonen

import os
import re
import gi
import signal
import subprocess
from subprocess import PIPE
gi.require_version('Gtk', '3.0')
gi.require_version('AppIndicator3', '0.1')
gi.require_version('Notify', '0.7')
from gi.repository import Gtk as gtk
from gi.repository import AppIndicator3 as appindicator
from gi.repository import Gio as gio
from gi.repository import Notify
import urllib
import pwd
import webbrowser
from time import sleep

username = pwd.getpwuid(os.getuid()).pw_name
hostname = subprocess.run("hostname",stdout=PIPE,shell=True).stdout.decode("utf-8").strip()
check_reboot = False
item_created = False

class IndicatorPlaces:
    BOOKMARKS_PATH = '/usr/share/hy-menu/hy-menu-items'

    def __init__(self):
        global hostname
        global check_reboot
        if os.path.isfile("/var/run/reboot-required"):
            self.ind = appindicator.Indicator.new("places","/usr/share/hy-menu/hy-menu-red.png", appindicator.IndicatorCategory.APPLICATION_STATUS)
            check_reboot = True
        else:
            self.ind = appindicator.Indicator.new("places","/usr/share/hy-menu/hy-menu.png", appindicator.IndicatorCategory.APPLICATION_STATUS)
        self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)

        if hostname.startswith('vdi-') or hostname.startswith('dx'):
            self.gpath = "/home/ad/P-drive"
            self.zpath = "/home/ad/home/" + username[:1] + "/" + username
        else:
            self.gpath = "sftp://pangolin.it.helsinki.fi/home/ad/P-drive"
            self.zpath = "sftp://pangolin.it.helsinki.fi/home/ad/home/" + username[:1] + "/" + username

        self.update_menu()
        return

    def create_menu_item(self, label):

        item = gtk.MenuItem()

        item.set_label(label)

        item.show()
        return item

    # Icons for different menu items
#    def get_bookmark_icon(self, path):
#        if path.endswith("helpdesk"):
#            icon_name = "help"
#        elif path.endswith("chat_index.html"):
#            icon_name = "user-available-panel"
#        elif path.startswith("vmware"):
#            icon_name = "vmware-view"
#        else:
#            f = gio.File.new_for_uri(path)
#            info = f.query_info('standard::icon',0,gio.Cancellable())
#            icon_name = info.get_icon().get_names()[0]
#        return icon_name

    # This creates the menu
    def update_menu(self, widget = None, data = None):
        global item_created
        global check_reboot
        global username
        try:
            bookmarks = open(self.BOOKMARKS_PATH).readlines()
        except IOError:
            bookmarks = []

        # Create menu
        menu = gtk.Menu()
        self.ind.set_menu(menu)

        item = self.create_menu_item("\U0001f4BB " + hostname)
        menu.append(item)

	# Show ip
#        if len(subprocess.check_output("hostname -I", shell=True)) > 17:
#            self.ips = subprocess.check_output("hostname -I", shell=True).decode('utf-8').split()
#            self.ip = self.ips[0] + " | " + self.ips[1]
#        else:
#            self.ip = subprocess.check_output("hostname -I", shell=True).strip().decode('utf-8')
##            self.hip = "IP address: "
#        item = self.create_menu_item("\U0001F4DE " + self.ip)
#        menu.append(item)

	# Ask for help
        item = self.create_menu_item("\U0001F4F6 Enable remote help" )
        item.connect("activate", self.ask_for_help)
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Add bookmarks to the menu from the /usr/share/hy-menu/hy-menu-items
        for bm in bookmarks:
            path, label = bm.strip().partition(' ')[::2]

            if not label:
                label = os.path.basename(os.path.normpath(path))
#            label = urllib.unquote(label)
            if label.startswith('VDI C'):
                item = self.create_menu_item("\U0001f427 " + label) #, self.get_bookmark_icon(path))
            elif label.startswith('VDI W'):
                item = self.create_menu_item("\U0001fa9f " + label) #, self.get_bookmark_icon(path))
            else:
                item = self.create_menu_item("\U0001f30d " + label) #, self.get_bookmark_icon(path))
            item.connect("activate", self.on_bookmark_click, path)

            # Append the item to menu
            # Check if the host is vdi machine --> disable vmware items.
            if hostname.startswith('vdi-') and path.startswith('vmware-view'):
                continue
            else:
                menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

	# Upgrade to the newest Cubbli
        item = self.create_menu_item("\U0001f503 Upgrade Cubbli version")
        item.connect("activate", self.RunProgram, "/usr/bin/check-reinstall")
        menu.append(item)

	# Check for software updates
        item = self.create_menu_item("\U0001F4BF Check for software updates")
        item.connect("activate", self.RunProgram, "/usr/bin/update-manager")
        menu.append(item)

	# Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # Computer info
        item = self.create_menu_item("\U0001f4a1 Computer Info")
        item.connect("activate", self.computerInfo)
        menu.append(item)

        # User info
        item = self.create_menu_item("\U0001fa9e User Info")
        item.connect("activate", self.userInfo)
        menu.append(item)

        # HY install printers
        item = self.create_menu_item("\U0001f5A8 HY Install Printers")
        item.connect("activate", self.InstallPrinter)
        menu.append(item)

        # Show separator
        item = gtk.SeparatorMenuItem()
        menu.append(item)

        # HY install software
#        item = self.create_menu_item("HY Install Software", "emblem-downloads")
#        item.connect("activate", self.InstallSoftware)
#        menu.append(item)

        # Create group directory menu item
        item = self.create_menu_item("\U0001f4BE IT Department group directories")
        item.connect("activate", self.on_netdrive_click, self.gpath)
        menu.append(item)

        # Create Z-drive menu item
        item = self.create_menu_item("\U0001f4BE Z-drive")
        item.connect("activate", self.on_netdrive_click, self.zpath)
        menu.append(item)

        # OneDrive
        item = self.create_menu_item("\U0001f4BE OneDrive")
        item.connect("activate", self.RunProgram, "/usr/bin/cubbli-rclone-setup open")
        menu.append(item)

        # If reboot required
        if check_reboot == True:
            item_r = self.create_menu_item("\U0001f534 Reboot required!")
            item_r.connect("activate", self.RunProgram, "/usr/sbin/menu-reboot.sh open")
            menu.append(item_r)
            item_created = True
        if check_reboot == False and item_created == True:
            for item_r in menu.get_children():
                menu.remove(item_r)
            item_created = False

        # Show the menu
        menu.show_all()
        return

    def RunProgram(self, widget, shellscript):
        subprocess.Popen(shellscript, shell = True)
        return

    # Open network drive with the default file browser
    def on_netdrive_click(self, widget, path):
        subprocess.Popen('gtk-launch $(xdg-mime query default inode/directory|sed s/\\.desktop//) ' + path, shell = True)
        return

    # Open clicked bookmark
    def on_bookmark_click(self, widget, path):
        if path.endswith("chat_index.html"):
            subprocess.Popen('/usr/bin/chromium-browser --window-size=420,540 --app=' + path, shell=True)
        else:
            subprocess.run(["/usr/bin/xdg-open", path])
        return

    def userInfo(self, item):
        subprocess.Popen('/usr/share/hy-menu/userinfo.py %s', shell = True)
        return

    def computerInfo(self, item):
        subprocess.Popen('/usr/share/hy-menu/computerinfo.py %s', shell = True)
        return

    def InstallPrinter(self, item):
        subprocess.Popen('/usr/share/hy-menu/hy-printers.py %s', shell = True)
        return

#    def InstallSoftware(self, item):
#        subprocess.Popen('/usr/bin/hy-software.py %s', shell = True)
#        return

    def on_bookmarks_changed(self, filemonitor, file, other_file, event_type):
        if event_type == gio.FILE_MONITOR_EVENT_CHANGES_DONE_HINT:
            print ('he-menu: bookmarks changed, updating menu...')
            self.update_menu()
        return

    def reboot_notification(self):
        global check_reboot
        Notify.init("Reboot required")
        notification = Notify.Notification.new("Reboot required!")
        if check_reboot == False:
            notification.show()
        check_reboot = True
        return

    def check_if_reboot_needed(self, filemonitor, f1, f2, event_type):
        global check_reboot
        if os.path.isfile("/var/run/reboot-required"):
            self.ind = appindicator.Indicator.new("places","/usr/share/hy-menu/hy-menu-red.png", appindicator.IndicatorCategory.APPLICATION_STATUS)
            self.reboot_notification()
        else:
            self.ind = appindicator.Indicator.new("places","/usr/share/hy-menu/hy-menu.png", appindicator.IndicatorCategory.APPLICATION_STATUS)
            check_reboot = False

        self.ind.set_status(appindicator.IndicatorStatus.ACTIVE)
        self.update_menu()
        return

    def ask_for_help(self, item):
        subprocess.Popen('nmcli con up "uh-vpn-allroute ($USER)" | nmcli con up "uh-vpn-remote-help ($USER)"', shell = True)
        sleep(3)
        subprocess.Popen('/usr/share/hy-menu/askforhelp.py', shell = True)
        return

if __name__ == "__main__":
    # Catch CTRL-C
    signal.signal(signal.SIGINT, lambda signal, frame: gtk.main_quit())

    # Run the indicator
    i = IndicatorPlaces()

    # Monitor bookmarks changes
    file = gio.File.new_for_path(i.BOOKMARKS_PATH)
    monitor = file.monitor_file(gio.FileMonitorFlags.NONE,None)
    monitor.connect("changed", i.on_bookmarks_changed)

    # Monitor if reboot needed
    file2 = gio.File.new_for_path("/var/run/reboot-required")
    monitor2 = file2.monitor_file(gio.FileMonitorFlags.NONE, None)
    monitor2.connect("changed", i.check_if_reboot_needed)
    monitor2.connect("changed", i.check_if_reboot_needed)

    # Main gtk loop
    gtk.main()
